// Package sd provides utilities related to service discovery. That includes the
// client-side loadbalancer pattern, where a microservice subscribes to a
// service discovery system in order to reach remote instances; as well as the
// registrator pattern, where a microservice registers itself in a service
// discovery system. Implementations are provided for most common systems.
package sd
