// Package lb implements the client-side load balancer pattern. When combined
// with a service discovery system of record, it enables a more decentralized
// architecture, removing the need for separate load balancers like HAProxy.
package lb
